
# Add functions to surgeo.db namespace
from surgeo.db.unsupress import reconstitute_data
from surgeo.db.db_setup_geocode import setup_geocode_table
from surgeo.db.db_setup_surname import setup_surname_table




import os


def folder_cleanup():
    '''Delete all .uf1 and .zip files in folder.

    Args:
        None
    Returns:
        None
    Raises:
        None

    '''

    # Created named tuple for organizing
    home_dir_path = os.path.expanduser("~")
    data_dir_path = os.path.join(home_dir_path, '.surgeo')
    file_list = os.listdir(data_dir_path)
    for filename in file_list:
        if filename == 'census.db':
            continue
        if filename == 'logo.gif':
            continue
        os.remove(os.path.join(data_dir_path,
                               filename))




# Add functions and error to surgeo.utilities namespace
from surgeo.utilities.folder_utility import setup_folder
from surgeo.utilities.folder_cleanup import folder_cleanup
from surgeo.utilities.parser import get_parser_args




import configparser
import os
import sys


def setup_folder(verbose):
    '''This method sets up a user folder and accompanying files.

    Args:
        verbose: whether the function prints updates to stdout
    Returns:
        None
    Raises:
        None

    '''

    if verbose is True:
        sys.stdout.write('Checking folder setup ... \t\t\t')
    home_dir_path = os.path.expanduser("~")
    data_dir_path = os.path.join(home_dir_path, '.surgeo')
    # Create file application data folder if needed.
    if not os.path.exists(data_dir_path):
        os.mkdir(data_dir_path)
    if verbose is True:
        sys.stdout.write('OK\n')



import surgeo

'''This is the basic model. Efficiency later.

   Where k is census block
   Where j is surname
   Where i is race (1 = Hispanic,
                    2 = White,
                    3 = Black,
                    4 = Asian or Pacific Islander,
                    5 = American Indian / Alaska Native,
                    6 = Multiracial)

   Where u(i,j,k) = p(i|j) * r(k|i):

                                         u(i,j,k)
   q(i|j,k) = ---------------------------------------------------------------
              u(1,j,k) + u(2,j,k) + u(3,j,k) + u(4,j,k) + u(5,j,k) + u(6,j,k)

   What this means is better explained in the BACKGROUND.txt file.

'''

# Race dict for all functions
races = ['Hispanic',
         'White',
         'Black',
         'Asian or Pacific Islander',
         'American Indian / Alaska Native',
         'Multiracial']
# Index starts at 1 instead of zero
race_dict = {item[0]: item[1] for item in enumerate(races, start=1)}


def run_model(zcta, surname, db):
    '''Takes zcta, surname. Returns data percentages.

    Args:
        zcta: int
        surname: text
        db: Sqlite3 database connection instance
    Returns:
        SurgeoResult instance
    Raises:
        None

    Takes zip code, surname, and a database instance. Returns 6 separate
    probabilities for each of the races above.

    For each (zcta, surname) pair, we must first get the probability for a
    specific race is based on surname. We must then get get the probability for
    a specific race based on location. We then take prob_surname for that race
    and multiply it by prob_location. We multiply these together to get a
    unified surname/geocode probability. We divide this probability by the
    probability of all the races together to give us our final probability.
    Note that if there is no surname availible, fixed percentages are applied
    that should apply for all non-represented names. The numbers for this
    particular statistic are fixed below, but the explanation is outside the
    scope of this program.

    '''

    #u(1,j,k)
    prob_hispanic = get_combined_prob(1, zcta, surname, db)
    #u(2,j,k)
    prob_white = get_combined_prob(2, zcta, surname, db)
    #u(3,j,k)
    prob_black = get_combined_prob(3, zcta, surname, db)
    #u(4,j,k)
    prob_api = get_combined_prob(4, zcta, surname, db)
    #u(5,j,k)
    prob_ai = get_combined_prob(5, zcta, surname, db)
    #u(6,j,k)
    prob_multi = get_combined_prob(6, zcta, surname, db)
    #denom = u(1,j,k) + u(2,j,k) + u(3,j,k) + u(4,j,k) + u(5,j,k) + u(6,j,k)
    prob_combined = (prob_hispanic +
                     prob_white +
                     prob_black +
                     prob_api +
                     prob_ai +
                     prob_multi)
    return surgeo.SurgeoResult(surname,
                               zcta,
                               '{0:f}'.format(prob_hispanic/prob_combined),
                               '{0:f}'.format(prob_white/prob_combined),
                               '{0:f}'.format(prob_black/prob_combined),
                               '{0:f}'.format(prob_api/prob_combined),
                               '{0:f}'.format(prob_ai/prob_combined),
                               '{0:f}'.format(prob_multi/prob_combined))


def get_combined_prob(race,
                      zcta,
                      surname,
                      db):
    '''This gets combined probability: u(i,j,k)

    Args:
        race: int
        zcta: text
        surname: text
        db: Sqlite3 database connection instance
    Returns:
        combined_prob: float
    Raises:
        None

    '''

    surname_prob = get_prob_surname(race, surname, db)
    zcta_prob = get_prob_zcta(race, zcta, db)
    combined_prob = surname_prob * zcta_prob
    return combined_prob


def get_prob_surname(race,
                     surname,
                     db):
    '''This gets probability of surname: p(i|j)

    Args:
        race: int
        surname: text
        db: Sqlite3 database connection instance
    Returns:
        combined_prob: float
    Raises:
        None

    '''

    cursor = db.cursor()
    cursor.execute('''SELECT pcthispanic, pctwhite, pctblack, pctapi,
                      pctaian, pct2prace FROM surname_data WHERE name=?''',
                   (surname,))
    fetched = cursor.fetchone()
    # csv in percentage form
    hispanic = fetched[0]/100
    white = fetched[1]/100
    black = fetched[2]/100
    api = fetched[3]/100
    ai = fetched[4]/100
    multiracial = fetched[5]/100
    # Sync race with probability
    prob_index = {1: hispanic,
                  2: white,
                  3: black,
                  4: api,
                  5: ai,
                  6: multiracial}
    return prob_index[race]


def get_prob_zcta(race,
                  zcta,
                  db):
    '''This gets probability of zcta: r(k|i).

    Args:
        race: int
        zcta: text
        db: Sqlite3 database connection instance
    Returns:
        prob_index: dict
    Raises:
        None

    '''

    # Add up all the blocks in the zip.
    cursor = db.cursor()
    count_hispanic = 0
    count_white = 0
    count_black = 0
    count_api = 0
    count_ai = 0
    count_multi = 0
    # First select a list of logical records for a given zip
    cursor.execute('''SELECT state, logical_record FROM
                      geocode_data WHERE zcta=?''', (zcta,))
    state, logical_record = cursor.fetchone()
    # For each of the results, get race results for logical record
    # Each state will likely have a logical record with that number, filter
    # State
    cursor.execute('''SELECT num_hispanic, num_white, num_black,
                      num_api, num_ai, num_multi FROM logical_race_data
                      WHERE logical_record=? AND state=?''',
                   (logical_record, state))
    row = cursor.fetchone()
    # hispanic, white, black, api, indian, multirace.
    count_hispanic = row[0]
    count_white = row[1]
    count_black = row[2]
    count_api = row[3]
    count_ai = row[4]
    count_multi = row[5]
    # total should come from data pop, not adding components together
    total = (count_hispanic + count_white + count_black + count_api +
             count_ai + count_multi)
    hispanic = count_hispanic / total
    white = count_white / total
    black = count_black / total
    api = count_api / total
    ai = count_ai / total
    multiracial = count_multi / total
    # Sync race with probability
    prob_index = {1: hispanic,
                  2: white,
                  3: black,
                  4: api,
                  5: ai,
                  6: multiracial}
    return prob_index[race]




import configparser
import decimal
import os
import sqlite3
import sys
import time
import traceback
import urllib.request
import zipfile


def setup_surname_table(verbose):
    '''This creates the surname database and does housekeeping.

    Args:
        verbose: True/False for whether function outputs info.
    Returns:
        None
    Raises:
        None

    This downloads a single census data file which gives the relative ethnic
    makeup for each individual name. It only includes names with over 100
    instances.

    '''

    if verbose is True:
        sys.stdout.write('Checking db existence ... \t\t\t')
    db_path = os.path.join(os.path.expanduser('~'),
                           '.surgeo',
                           'census.db')
    zipfile_path = os.path.join(os.path.expanduser('~'),
                                '.surgeo',
                                'data.zip')
    csv_path = os.path.join(os.path.expanduser('~'),
                            '.surgeo',
                            'census_data.csv')
    if not os.path.exists(db_path):
        if verbose is True:
            sys.stdout.write('\nCreating db ... \n')
        config_path = os.path.join(os.path.expanduser('~'),
                                   '.surgeo',
                                   'configuration.txt')
        parser_instance = configparser.ConfigParser()
        parser_instance.read(config_path)
        url = 'http://www.census.gov/genealogy/www/data/2000surnames/names.zip'
        # No try block for url. If it fails, no point in continuing.
        #site = urllib.request.urlopen(url)
        #length_in_bytes = int(site.info()['Content-Length'])
        if verbose is True:
            urllib.request.urlretrieve(url,
                                       zipfile_path,
                                       download_bar)
        else:
            urllib.request.urlretrieve(url,
                                       zipfile_path)
        if verbose is True:
            sys.stdout.write('\n')
        # Done
        if verbose is True:
            sys.stdout.write('Re-checking folder setup ... \t\t\t')
            sys.stdout.write('OK\n')
            # Zip file
            sys.stdout.write('Extracting zip file ... \t\t\t')
        time.sleep(0)
        with zipfile.ZipFile(zipfile_path, 'r') as f:
            data = f.read('app_c.csv')
        # Write zip data to csv
        time.sleep(0)
        with open(csv_path, 'wb+') as f:
            f.write(data)
        if verbose is True:
            sys.stdout.write('OK\n')
        # Write line by line of csv to db
        csv_lines = open(csv_path, 'r').readlines()
        time.sleep(0)
        csv_length = len(csv_lines)
        line_count = 0
        # Create DB
        connection = sqlite3.connect(db_path)
        cursor = connection.cursor()
        try:
            cursor.execute('''CREATE TABLE IF NOT EXISTS surname_data (id
                              INTEGER PRIMARY KEY, name TEXT, rank INTEGER,
                              count INTEGER, prop1000K REAL, cum_prop100K REAL,
                              pctwhite REAL, pctblack REAL, pctapi REAL,
                              pctaian REAL, pct2prace REAL,
                              pcthispanic REAL)''')
            cursor.execute('''CREATE INDEX IF NOT EXISTS name_index ON
                              surname_data(name)''')
            # Strip csv header.
            for line in csv_lines[1:]:
                time.sleep(0)
                cursor.execute('''INSERT into surname_data VALUES (NULL, ?, ?,
                                  ?, ?, ?, ?, ?, ?, ?, ?, ?)''',
                               (line.split(',')))
                line_count += 1
                percent = int(decimal.Decimal(line_count) /
                              decimal.Decimal(csv_length) * 100)
                if verbose is True:
                    try:
                        last_written_percent
                    except NameError:
                        last_written_percent = 0
                    # Prevent flicker. Only update when percentage changes
                    if percent > last_written_percent:
                        sys.stdout.write('\rItems written: {}%'.
                                         format(percent))
                        last_written_percent = percent
            if verbose is True:
                sys.stdout.write('\rItems written: 100%')
                sys.stdout.write('\n')
                sys.stdout.write('Db write cleanup ... \t\t\t\t')
            connection.commit()
            connection.close()
            if verbose is True:
                sys.stdout.write('OK\n')
        except sqlite3.Error as e:
            traceback.print_exc()
            connection.rollback()
            connection.close()
            raise e
    else:
        if verbose is True:
            sys.stdout.write('OK\n')


def download_bar(block_count, block_size, total_size):
    '''Report hook for use in db_setup_in_ram function.'''
    percentage = int((decimal.Decimal(block_count) *
                      decimal.Decimal(block_size) /
                      decimal.Decimal(total_size) * 100))
    sys.stdout.write('\rDownloading surname data: {}%'.format(str(percentage)))
    time.sleep(0)
